/*
 * DS18B20.c
 *
 * Created: 30.07.2012 22:04:48
 *  Author: hberg
 */ 

#define F_CPU 16000000

#include <avr/io.h>
#include <util/delay.h>
#include <stdio.h>
#include "ser.h"

#define D1W_DDR		DDRC
#define D1W_PORT	PORTC
#define D1W_PIN		PINC
#define D1W_DQ		PINC3

void d1w_init() {
	
	// set DDR to output
	D1W_DDR |= (1 << D1W_DQ);
	
	// put signal low
	D1W_PORT &= (1 << D1W_DQ);
	
	// wait at least 480us
	_delay_us(500);
	
	// release by DDR to input
	D1W_DDR &= ~(1 << D1W_DQ);
	
	// wait at least 60+240us
	_delay_us(310);
}

void d1w_write(uint8_t data) {
	
	for (uint8_t i = 0; i < 8; i++) {
		
		// set DDR to output
		D1W_DDR |= (1 << D1W_DQ);
		
		// write 1
		if (data & (1 << i)) {
			
			// put signal low
			D1W_PORT &= ~(1 << D1W_DQ);
			
			// relase within 15us
			_delay_us(5);
			
			// release by DDR to input
			D1W_DDR &= ~(1 << D1W_DQ);
			
			// wait at least 60us
			_delay_us(60);
			
		// write 0
		} else {
			
			// put signal low
			D1W_PORT &= ~(1 << D1W_DQ);
			
			// hold down for at least 60us
			_delay_us(65);
			
			// release by DDR to input
			D1W_DDR &= ~(1 << D1W_DQ);
		}
		
		// wait at least 1us (recovery time)
		_delay_us(1);
	}
}

uint8_t d1w_read() {
	
	uint8_t data = 0x00;
	
	for (uint8_t i = 0; i < 8; i++) {
		
		// set DDR to output
		D1W_DDR |= (1 << D1W_DQ);
	
		// put signal low
		D1W_PORT &= ~(1 << D1W_DQ);
	
		// hold down for at least 1us
		_delay_us(1);
	
		// release by DDR to input
		D1W_DDR &= ~(1 << D1W_DQ);
	
		// wait
		_delay_us(10);
	
		// read 1 (high)
		if (D1W_PIN & (1 << D1W_DQ)) {
			data |= (1 << i);
			
		//  read 0 (low)
		} else {
			
			data &= ~(1 << i);
			
			// wait until high
			//while (D1W_PIN & (1 << D1W_DQ));
		}
		
		// total slot at least 60us
		_delay_us(55);
	}
	
	return data;
}

#define D1W_MATCH_ROM			0x55
#define D1W_SKIP_ROM			0xCC
#define D1W_CONVERT_T			0x44
#define D1W_READ_SCRATCHPAD		0xbe

int main(void)
{
	// init serial
	ser_init();
	ser_set_stdout();
	
    while(1)
    {
		d1w_init();
		d1w_write(D1W_SKIP_ROM); // skip rom
		d1w_write(D1W_CONVERT_T);

		// wait 750ms
		_delay_ms(750);
	
		d1w_init();
		d1w_write(D1W_SKIP_ROM); // skip rom
		d1w_write(D1W_READ_SCRATCHPAD);

		uint8_t tempLB = d1w_read();
		uint8_t tempHB = d1w_read();
		uint16_t temp = ((uint16_t)tempHB << 8 ) | tempLB;
		printf("%d\n", temp);
		
		_delay_ms(1000);
    }
}